package me.unurled.raxen.manager.server;

import static me.unurled.raxen.utils.Utils.debug;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Objects;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import lombok.Getter;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.resourcepack.ResourcePack;
import me.unurled.raxen.components.resourcepack.hosting.Hosting;
import me.unurled.raxen.components.resourcepack.hosting.PackHoster;
import org.bukkit.configuration.file.FileConfiguration;

public class ResourcePackManager {

    private Raxen main;

    @Getter
    private ResourcePack resourcePack;

    private FileConfiguration config;

    private UUID uuid;
    @Getter
    private String url;
    @Getter
    private String sha1;
    private File resourcePackFile;

    @Getter
    private boolean useRP;

    public ResourcePackManager(Raxen main) {
        this.main = main;
        this.resourcePack = new ResourcePack(main, this);
        this.config = main.getConfig();
        this.useRP = this.config.getBoolean("useResourcePack", false);
        prepareFolder();
        resourcePackFile = packageRp();
        if (resourcePackFile != null) {
            enable();
        } else {
            useRP = false;
        }
        if (useRP) {
            upload();
        }
    }

    /**
     * Create all folders for the resource pack
     */
    private void prepareFolder() {
        // Create main folder3
        File rp = new File(main.getDataFolder(), "resoucepack");
        rp.mkdirs();

        File minecraft = new File(rp, "minecraft");
        minecraft.mkdirs();

        // Create mcmeta file
        String content = """
             {
                "pack": {
                    "pack_format": 12,
                    "description": "§bRaxen - By §6unurled"
                }
            }""";
        try {
            File file = new File(minecraft, "pack.mcmeta");

            FileWriter fileWriter = new FileWriter(file);

            BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

            bufferedWriter.write(content);

            bufferedWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }

        // Create texture folder
        File texture = new File(minecraft, "textures");
        texture.mkdirs();
        new File(texture, "item").mkdirs();
        new File(texture, "block").mkdirs();

        // Create model folder
        File model = new File(minecraft, "models");
        model.mkdirs();
        new File(model, "item").mkdirs();
        new File(model, "block").mkdirs();

        // Create font folder
        File font = new File(minecraft, "font");
        font.mkdirs();

        // Create sound folder
        File sound = new File(minecraft, "sounds");
        sound.mkdirs();
    }

    private File packageRp() {
        String zipFilePath = main.getDataFolder() + "/resourcepack.zip";
        String folderPath = main.getDataFolder() + "/resourcepack";
        try {
            zipFolder(folderPath, zipFilePath);
            return new File(zipFilePath);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Method to enable the ressource pack
     */
    private void enable() {
        if (useRP) {
            if (config.getString("resource_pack_url") == null ||
                Objects.equals(config.getString("resource_pack_url"), "")) {
                debug(main, "Resource pack not enabled.");
                useRP = false;
            } else {
                debug(main, "Ressource Pack enabled.");
                if (config.contains("resource_pack_uuid") &&
                    config.getString("resource_pack_uuid") != null &&
                    !Objects.equals(config.getString("resource_pack_uuid"),
                        "00000000-0000-0000-0000-000000000000")) {
                    uuid = UUID.fromString(config.getString("resource_pack_uuid"));
                } else {
                    uuid = UUID.randomUUID();
                    config.set("resource_pack_uuid", uuid.toString());
                    main.saveConfig();
                }
                url = config.getString("resource_pack_url") + "/download/" + uuid.toString();
            }
        } else {
            debug(main, "Resource pack not enabled.");
        }
    }

    private static void zipFolder(String folderPath, String zipFilePath) throws IOException {
        FileOutputStream fos = new FileOutputStream(zipFilePath);
        ZipOutputStream zos = new ZipOutputStream(fos);

        File folder = new File(folderPath);
        zipFiles(folder, folder.getName(), zos);

        zos.close();
        fos.close();
    }

    private static void zipFiles(File folder, String parentFolder, ZipOutputStream zos)
        throws IOException {
        byte[] buffer = new byte[1024];
        if (folder.listFiles() == null) {
            return;
        }

        for (File file : folder.listFiles()) {
            if (file.isDirectory()) {
                zipFiles(file, parentFolder + "/" + file.getName(), zos);
                continue;
            }

            FileInputStream fis = new FileInputStream(file);

            zos.putNextEntry(new ZipEntry(parentFolder + "/" + file.getName()));

            int length;
            while ((length = fis.read(buffer)) > 0) {
                zos.write(buffer, 0, length);
            }

            zos.closeEntry();
            fis.close();
        }
    }

    private void upload() {
        resourcePackFile = packageRp();
        if (resourcePackFile != null) {
            Hosting host = createHosting();
            host.upload(resourcePackFile);
            sha1 = host.getOriginalSHA1();
        }
    }

    private Hosting createHosting() {
        return new PackHoster(url);
    }
}
